@@ -6,7 +6,11 @@ import logging
 import time
 
 from dbnd._core.errors import DatabandError
-from dbnd._core.utils.http.reliable_http_client import ReliableHttpClient
+from dbnd._core.utils.http.constants import LINEAR_RETRY, LINEAR_RETRY_ANY_ERROR
+from dbnd._core.utils.http.reliable_http_client import (
+    HttpClientException,
+    ReliableHttpClient,
+)
 from dbnd._core.utils.http.retry_policy import get_retry_policy
 
 
@@ -20,40 +24,52 @@ class LivyBatchClient(object):
     """A Livy-specific Http client which wraps the normal ReliableHttpClient. Propagates
     HttpClientExceptions up."""
 
-    def __init__(self, http_client, endpoint):
+    def __init__(self, http_client, safe_retry_policy, endpoint):
         self.endpoint = endpoint
         self._http_client = http_client
+        self._safe_retry_policy = safe_retry_policy
 
     @staticmethod
-    def from_endpoint(endpoint, ignore_ssl_errors=False):
+    def from_endpoint(
+        endpoint, status_code_retries, ignore_ssl_errors=False,
+    ):
         headers = {"Content-Type": "application/json"}
         custom_headers = {}
-
         headers.update(custom_headers)
-        retry_policy = LivyBatchClient._get_retry_policy()
         return LivyBatchClient(
-            ReliableHttpClient(
-                endpoint, headers, retry_policy, ignore_ssl_errors=ignore_ssl_errors
+            http_client=ReliableHttpClient(
+                endpoint,
+                headers,
+                get_retry_policy("LivyBatch'", policy=LINEAR_RETRY),
+                ignore_ssl_errors=ignore_ssl_errors,
             ),
-            endpoint,
+            safe_retry_policy=get_retry_policy(
+                "LivyBatch'",
+                policy=LINEAR_RETRY_ANY_ERROR,
+                max_retries=status_code_retries,
+            ),
+            endpoint=endpoint,
         )
 
     def post_batch(self, properties):
         return self._http_client.post("/batches", [201], properties).json()
 
     def get_batch(self, batch_id):
-        return self._http_client.get(self._batch_url(batch_id), [200]).json()
-
-    def get_batch_state(self, batch_id):
-        return self._http_client.get(self._batch_url(batch_id), [200]).json()
+        return self._http_client.get(
+            self._batch_url(batch_id), [200], retry_policy=self._safe_retry_policy
+        ).json()
 
     def get_all_batch_logs(self, batch_id, from_line=0):
         return self._http_client.get(
-            self._batch_url(batch_id) + "/log?from=%s" % from_line, [200]
+            self._batch_url(batch_id) + "/log?from=%s" % from_line,
+            [200],
+            retry_policy=self._safe_retry_policy,
         ).json()
 
     def get_batches(self):
-        return self._http_client.get("/batches", [200]).json()
+        return self._http_client.get(
+            "/batches", [200], retry_policy=self._safe_retry_policy
+        ).json()
 
     def delete_batch(self, batch_id):
         self._http_client.delete(self._batch_url(batch_id), [200, 404])
@@ -65,12 +81,7 @@ class LivyBatchClient(object):
     def _batch_url(batch_id):
         return "/batches/{}".format(batch_id)
 
-    @staticmethod
-    def _get_retry_policy():
-        return get_retry_policy("LivyBatch'")
-
-        # Function to help track the progress of the scala code submitted to Apache Livy
-
+    # Function to help track the progress of the scala code submitted to Apache Livy
     def _print_status(self, response):
         logger.info
 
@@ -80,6 +91,7 @@ class LivyBatchClient(object):
         )
 
     def track_batch_progress(self, batch_id, status_reporter=None):
+        logger.debug("Beginning to track batch %s", batch_id)
         status_reporter = status_reporter or self._default_status_reporter
 
         # Poll the status of the submitted scala code
